@@ -4,6 +4,7 @@ import logging
 import os
 import re
 import shutil
+import threading
 import time
 import uuid
 from json import loads
@@ -12,15 +13,15 @@ from os.path import join
 from time import localtime
 from time import sleep
 from types import MethodType
-from typing import Union, IO, List
-from urllib.parse import urljoin
+from typing import Union, IO, List, Callable
+from urllib.parse import urljoin, quote
 
 import requests
 from lxml.etree import HTML
 from lxml.html import HtmlElement
 
 from spider.cache import SqliteCache
-from .utils import general_response_pipeline, get_random_header, limit_text
+from .utils import general_response_pipeline, get_random_header, limit_text, random_sleeper
 
 
 # 改变脚本的工作目录
@@ -28,8 +29,6 @@ from .utils import general_response_pipeline, get_random_header, limit_text
 # FILE_DIR = os.path.dirname(os.path.abspath(__file__))
 # os.chdir(FILE_DIR)
 
-# todo prepare to delete this function
-
 
 class FormatFilter(logging.Filter):
 
@@ -256,6 +255,11 @@ class Spider:
             self.status_code = response.status_code
             self.__html = None
             self.__json = None
+            self.__title = None
+
+        @property
+        def history(self):
+            return self.response.history
 
         @property
         def content(self):
@@ -312,16 +316,54 @@ class Spider:
         def ok(self):
             return self.response.ok
 
+        @property
+        def title(self):
+            if not self.__title:
+                self.__title = self.xpath('//title/text()')[0]
+            return self.__title
+
+        def __repr__(self):
+            return '<Response[{}] [{}]>'.format(
+                self.response.status_code,
+                self.title
+            )
+
+        def __str__(self):
+            return self.__repr__()
+
     def __init__(self):
         self.headers_generator = get_random_header
         # self.cache = JsonCache()
         self.cache = SqliteCache()
         self.session = requests.session()
-        self.sep_time = 1
+        self.sleeper = random_sleeper(5, 10)
         self.response_pipeline = general_response_pipeline
         self.request_pipeline = None
+        # todo proxy_generator
+        self.proxy_generator = None
+        self.__encoding = None
         self.update_headers()
 
+    def set_sleeper(self, sleeper: Callable):
+        self.sleeper = sleeper
+
+    def set_header_generator(self, header_generator):
+        self.headers_generator = header_generator
+
+    def set_response_pipeline(self, response_pipeline):
+        self.response_pipeline = response_pipeline
+
+    def set_request_pipeline(self, request_pipeline):
+        self.request_pipeline = request_pipeline
+
+    @property
+    def encoding(self):
+        return self.__encoding
+
+    @encoding.setter
+    def encoding(self, encoding):
+        self.__encoding = encoding
+
     def set_cookie(self, cookie: str):
         """设置cookie
         :param cookie: 一个 `cookie` 字符串
@@ -332,14 +374,11 @@ class Spider:
             cookiejar=None,
             overwrite=True)
 
-    def set_sep_time(self, sep_time):
-        self.sep_time = sep_time
-
-    def __get_or_post(self, handle, *args, **kwargs) -> Union[Response, requests.Response, object]:
+    def lunch(self, handle: [requests.get, requests.post], *args, **kwargs) -> \
+            Union[Response, requests.Response, object]:
         """
 
         :param handle 处理请求的一个函数
-        :param args `url`的各个路径:
         :param kwargs: 包含`requests`库所有选项
         :keyword alive_time: Union[datetime, int]缓存存活日期
         :keyword cache: 是否使用缓存
@@ -351,9 +390,11 @@ class Spider:
         kwargs.setdefault('alive_time', datetime.datetime.now() + datetime.timedelta(days=3))
         kwargs.setdefault('cache', True)
         kwargs.setdefault('timeout', (5, 20))
+        kwargs.setdefault('retry', 3)
         alive_time = kwargs.pop('alive_time')
-        sep_time = kwargs.get('sep_time', self.sep_time)
+        sep_time = kwargs.get('sep_time', self.sleeper)
         cache_enable = kwargs.pop('cache')
+        retry = kwargs.pop('retry')
 
         url = ''
         for each in args:
@@ -368,7 +409,6 @@ class Spider:
         if self.cache.is_cached(url, ignore_date=is_force_cache) and cache_enable:
             logger.debug('从缓存: {} <- {}', limit_text(url, 200), '文件')
             return self.cache.from_cache(url, force=is_force_cache)
-        logger.info('下载: {}', limit_text(url, 200))
 
         if cache_enable == Spider.DISABLE_CACHE:
             # 如果禁用这个url的缓存, 则将之从缓存文件删除
@@ -377,17 +417,32 @@ class Spider:
         class HTTPBadCodeError(RuntimeError):
             ...
 
-        retry = 3
+        # 发送请求
+        # 显示logger文案
+        if 'params' in kwargs:
+            params = kwargs.get('params')
+            tmp = ['{}={}'.format(k, quote(v)) for k, v in params.items()]
+            logger.info('请求: {}', limit_text(url + '?' + '&'.join(tmp), 200))
+        else:
+            logger.info('请求: {}', limit_text(url, 200))
+
+        retry = retry
         while retry:
             try:
                 # 间隔时间
                 response = self.Response(handle(url, **kwargs))
-                sleep(sep_time)
-                if len(response.response.history) > 1:
-                    logger.debug('===重定向历史===\n{}', '\n'.join([each.url for each in response.response.history]))
+                if isinstance(sep_time, Callable):
+                    sep_time()
+                else:
+                    sleep(sep_time)
+                if len(response.response.history) >= 1:
+                    history = [each.url for each in response.response.history]
+                    history.append(response.url)
+                    logger.debug('===发生页面重定向===\n{}',
+                                 '->'.join(['[{}]'.format(i) for i in history]))
                 if response.response.ok:
                     if cache_enable == Spider.ENABLE_CACHE:
-                        self.cache.cache(url, response, alive_time)
+                        self.cache.cache(response.url, response, alive_time)
                 else:
                     raise HTTPBadCodeError(f'坏HTTP响应', response)
                 return response
@@ -421,7 +476,10 @@ class Spider:
         :return: Union[Response, requests.Response, object]
         """
         # 获取`alive_time`, `url`参数
-        return self.__get_or_post(self.session.get, *args, **kwargs)
+        resp = self.lunch(self.session.get, *args, **kwargs)
+        if self.response_pipeline:
+            resp = self.response_pipeline(self, resp)
+        return resp
 
     def post(self, *args, **kwargs) -> Response:
         """获取网页
@@ -434,7 +492,10 @@ class Spider:
 
         :return: Union[Response, requests.Response, object]
         """
-        return self.__get_or_post(self.session.post, *args, **kwargs)
+        resp = self.lunch(self.session.post, *args, **kwargs)
+        if self.response_pipeline:
+            resp = self.response_pipeline(self, resp)
+        return resp
 
     def update_headers(self):
         """调用`self.headers_generator`来更新头"""
@@ -445,9 +506,27 @@ class Spider:
         self.session.close()
 
 
-def quick_start():
-    ...
+local = threading.local()
 
 
 def get_spider():
-    ...
+    if 'spider' not in local:
+        local.spider = Spider()
+    return local.spider
+
+
+def gs():
+    """qs() == get_spider()"""
+    return get_spider()
+
+
+def get(*args, **kwargs):
+    gs().get(*args, **kwargs)
+
+
+def post(*args, **kwargs):
+    gs().post(*args, **kwargs)
+
+
+def close():
+    gs().close()
