@@ -1,4 +1,7 @@
 import logging
+import random
+from time import sleep
+from typing import Iterable
 
 from fake_useragent import UserAgent
 from peewee import SqliteDatabase
@@ -20,11 +23,12 @@ def get_random_header():
     return {'User-Agent': str(UserAgent().random)}
 
 
-def limit_text(s: str, max_len):
+def limit_text(s: str, max_len, replace='...'):
     """文本太长自动打省略号"""
     s_len = len(s)
-    if s_len + 3 > max_len:
-        return s[:int(max_len / 2)] + '...' + s[-int(max_len / 2):]
+    replace_len = len(replace)
+    if s_len + replace_len > max_len:
+        return s[:int(max_len / 2)] + replace + s[-int(max_len / 2):]
     else:
         return s
 
@@ -37,7 +41,9 @@ def elem_tostring(elem):
 
 
 def good_dirname(string: str) -> str:
+    """一个好目录名字"""
     string.replace('\n', '').replace('\t', '').replace(' ', '')
+    string = limit_text(string, 60, '___')
     return string
 
 
@@ -45,3 +51,15 @@ def get_sqlite_db(db_name='db.sqlite'):
     db = SqliteDatabase(db_name)
     logger.info('创建数据库[{}]'.format(db_name))
     return db
+
+
+def random_sleeper(a, b):
+    def sleeper():
+        sleep(random.randint(a, b))
+
+    return sleeper
+
+
+class ProxyGeneratorBase:
+    def __call__(self, *args, **kwargs) -> Iterable:
+        ...
